@@ -1,21 +1,21 @@
-/* $Id: filebuffer.cc,v 1.32 2004/06/16 22:37:46 daniel Exp $
+/*
+ * Copyright (c) 2002-2007  Daniel Elstner  <daniel.kitta@gmail.com>
  *
- * Copyright (c) 2004  Daniel Elstner  <daniel.elstner@gmx.net>
+ * This file is part of regexxer.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License VERSION 2 as
- * published by the Free Software Foundation.  You are not allowed to
- * use any other version of the license; unless you got the explicit
- * permission from the author to do so.
+ * regexxer is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * regexxer is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with regexxer; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "filebuffer.h"
@@ -33,13 +33,11 @@
 
 #include <config.h>
 
-
 namespace
 {
 
 enum { PULSE_INTERVAL = 128 };
 
-
 class RegexxerTags : public Gtk::TextTagTable
 {
 public:
@@ -70,7 +68,7 @@ RegexxerTags::RegexxerTags()
 {
   error_message->property_wrap_mode()          = Gtk::WRAP_WORD;
   error_message->property_justification()      = Gtk::JUSTIFY_CENTER;
-  error_message->property_pixels_above_lines() = 10;
+  error_message->property_pixels_above_lines() = 16;
 
   error_title->property_scale() = Pango::SCALE_X_LARGE;
 
@@ -109,7 +107,6 @@ Glib::RefPtr<RegexxerTags> RegexxerTags::instance()
 
 } // anonymous namespace
 
-
 namespace Regexxer
 {
 
@@ -142,7 +139,6 @@ FileBuffer::ScopedLock::~ScopedLock()
   buffer_.locked_ = false;
 }
 
-
 /**** Regexxer::FileBuffer::ScopedUserAction *******************************/
 
 class FileBuffer::ScopedUserAction
@@ -160,7 +156,6 @@ public:
   ~ScopedUserAction() { buffer_.end_user_action(); }
 };
 
-
 /**** Regexxer::FileBuffer *************************************************/
 
 FileBuffer::FileBuffer()
@@ -198,9 +193,9 @@ FileBuffer::create_with_error_message(const Glib::RefPtr<Gdk::Pixbuf>& pixbuf,
 
   iterator pend = buffer->insert_pixbuf(buffer->end(), pixbuf);
 
-  Glib::ustring title = "\302\240";
-  // TRANSLATORS: Please replace space by U+00A0 NO-BREAK SPACE.
-  title += _("Can't read file:");
+  Glib::ustring title = "\302\240"; // U+00A0 NO-BREAK SPACE
+
+  title += _("Can\342\200\231t read file:");
   title += '\n';
 
   pend = buffer->insert_with_tag(pend, title, tagtable->error_title);
@@ -259,7 +254,8 @@ bool FileBuffer::in_user_action() const
  * If multiple is false then every line is matched only once, otherwise
  * multiple matches per line will be found (like modifier /g in Perl).
  */
-int FileBuffer::find_matches(Pcre::Pattern& pattern, bool multiple)
+int FileBuffer::find_matches(Pcre::Pattern& pattern, bool multiple,
+                             const sigc::slot<void, int, const Glib::ustring&>& feedback)
 {
   ScopedLock lock (*this);
 
@@ -296,9 +292,9 @@ int FileBuffer::find_matches(Pcre::Pattern& pattern, bool multiple)
         return match_count_;
       }
 
-      const int capture_count = pattern.match(
-          subject, offset, (last_was_empty) ? Pcre::NOT_EMPTY : Pcre::MatchOptions(0));
-
+      const int capture_count =
+        pattern.match(subject, offset, (last_was_empty) ? Pcre::ANCHORED | Pcre::NOT_EMPTY
+                                                        : Pcre::MatchOptions(0));
       if (capture_count <= 0)
       {
         if (last_was_empty && unsigned(offset) < subject.bytes())
@@ -316,7 +312,7 @@ int FileBuffer::find_matches(Pcre::Pattern& pattern, bool multiple)
       ++match_count_;
       ++original_match_count_;
 
-      const std::pair<int,int> bounds = pattern.get_substring_bounds(0);
+      const std::pair<int, int> bounds = pattern.get_substring_bounds(0);
 
       iterator start = line;
       iterator stop  = line;
@@ -332,6 +328,9 @@ int FileBuffer::find_matches(Pcre::Pattern& pattern, bool multiple)
 
       apply_tag(tagtable->match, start, stop);
 
+      if (offset == 0 && feedback)
+        feedback(line.get_line(), subject);
+
       last_was_empty = (bounds.first == bounds.second);
       offset = bounds.second;
     }
@@ -518,7 +517,7 @@ void FileBuffer::decrement_stamp()
 
 void FileBuffer::undo_remove_match(const MatchDataPtr& match, int offset)
 {
-  const std::pair<MatchSet::iterator,bool> pos = match_set_.insert(match);
+  const std::pair<MatchSet::iterator, bool> pos = match_set_.insert(match);
   g_return_if_fail(pos.second);
 
   const iterator start = get_iter_at_offset(offset);
@@ -625,11 +624,11 @@ void FileBuffer::on_erase(const FileBuffer::iterator& rbegin, const FileBuffer::
       signal_preview_line_changed.queue();
   }
 
-  const Glib::RefPtr<RegexxerTags> tagtable = RegexxerTags::instance();
+  const Glib::RefPtr<const Gtk::TextTag> tag_match = RegexxerTags::instance()->match;
 
-  if (!rbegin.starts_line() && rbegin.has_tag(tagtable->match))
+  if (!rbegin.starts_line() && rbegin.has_tag(tag_match))
   {
-    g_return_if_fail(!rbegin.ends_tag(tagtable->match)); // just to be sure...
+    g_return_if_fail(!rbegin.ends_tag(tag_match)); // just to be sure...
 
     int backward_chars = 0;
     int match_length = -1;
@@ -649,7 +648,7 @@ void FileBuffer::on_erase(const FileBuffer::iterator& rbegin, const FileBuffer::
           match_length = std::max(match_length, match_data->length);
       }
     }
-    while (match_length < 0 && !pos.starts_line() && !pos.toggles_tag(tagtable->match));
+    while (match_length < 0 && !pos.starts_line() && !pos.toggles_tag(tag_match));
 
     if (match_length > backward_chars)
       remove_match_at_iter(pos);
@@ -916,4 +915,3 @@ void FileBuffer::notify_weak_undos()
 }
 
 } // namespace Regexxer
-
