@@ -1,21 +1,21 @@
-/* $Id: filebuffer.h,v 1.15 2004/05/17 17:21:04 daniel Exp $
+/*
+ * Copyright (c) 2002-2007  Daniel Elstner  <daniel.kitta@gmail.com>
  *
- * Copyright (c) 2002  Daniel Elstner  <daniel.elstner@gmx.net>
+ * This file is part of regexxer.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License VERSION 2 as
- * published by the Free Software Foundation.  You are not allowed to
- * use any other version of the license; unless you got the explicit
- * permission from the author to do so.
+ * regexxer is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * regexxer is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with regexxer; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef REGEXXER_FILEBUFFER_H_INCLUDED
@@ -50,7 +50,8 @@ public:
   bool is_freeable() const;
   bool in_user_action() const;
 
-  int find_matches(Pcre::Pattern& pattern, bool multiple);
+  int find_matches(Pcre::Pattern& pattern, bool multiple,
+                   const sigc::slot<void, int, const Glib::ustring&>& feedback);
 
   int get_match_count() const;
   int get_match_index() const;
@@ -77,7 +78,7 @@ public:
   sigc::signal<void>                signal_bound_state_changed;
   Util::QueuedSignal                signal_preview_line_changed;
   sigc::signal<bool>                signal_pulse;
-  sigc::signal<void,UndoActionPtr>  signal_undo_stack_push;
+  sigc::signal<void, UndoActionPtr> signal_undo_stack_push;
 
 protected:
   FileBuffer();
@@ -95,7 +96,7 @@ private:
   class ScopedLock;
   class ScopedUserAction;
 
-  typedef std::set<MatchDataPtr,MatchDataLess>      MatchSet;
+  typedef std::set<MatchDataPtr, MatchDataLess>     MatchSet;
   typedef std::stack<FileBufferActionRemoveMatch*>  WeakUndoStack;
 
   MatchSet            match_set_;
@@ -126,4 +127,3 @@ private:
 } // namespace Regexxer
 
 #endif /* REGEXXER_FILEBUFFER_H_INCLUDED */
-
