@@ -1,21 +1,21 @@
-/* $Id: filetree.cc,v 1.74 2004/07/05 22:43:05 daniel Exp $
+/*
+ * Copyright (c) 2002-2007  Daniel Elstner  <daniel.kitta@gmail.com>
  *
- * Copyright (c) 2002  Daniel Elstner  <daniel.elstner@gmx.net>
+ * This file is part of regexxer.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License VERSION 2 as
- * published by the Free Software Foundation.  You are not allowed to
- * use any other version of the license; unless you got the explicit
- * permission from the author to do so.
+ * regexxer is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * regexxer is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with regexxer; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "filetree.h"
@@ -25,18 +25,17 @@
 #include "stringutils.h"
 #include "translation.h"
 
+#include <gmodule.h>
+#include <glibmm.h>
 #include <gconfmm/client.h>
 #include <gtkmm/stock.h>
 
 #include <config.h>
 
-using namespace Regexxer::FileTreePrivate;
-
-
-namespace
-{
-
-extern "C"
+/*
+ * Custom widget creation function for libglade.
+ */
+extern "C" G_MODULE_EXPORT
 GtkWidget* regexxer_create_file_tree(char*, char*, char*, int, int)
 {
   try
@@ -51,12 +50,11 @@ GtkWidget* regexxer_create_file_tree(char*, char*, char*, int, int)
   }
 }
 
-} // anonymous namespace
-
-
 namespace Regexxer
 {
 
+using namespace Regexxer::FileTreePrivate;
+
 /**** Regexxer::FileTree ***************************************************/
 
 FileTree::FileTree()
@@ -66,6 +64,7 @@ FileTree::FileTree()
   sum_matches_    (0)
 {
   using namespace Gtk;
+  using sigc::mem_fun;
 
   set_model(treestore_);
   const FileTreeColumns& model_columns = FileTreeColumns::instance();
@@ -74,8 +73,8 @@ FileTree::FileTree()
   treestore_->set_sort_func(model_columns.collatekey, &collatekey_sort_func);
   treestore_->set_sort_column(TreeStore::DEFAULT_SORT_COLUMN_ID, SORT_ASCENDING);
 
-  treestore_->signal_rows_reordered().connect(
-      sigc::mem_fun(*this, &FileTree::on_treestore_rows_reordered));
+  treestore_->signal_rows_reordered()
+      .connect(mem_fun(*this, &FileTree::on_treestore_rows_reordered));
 
   {
     Column *const column = new Column(_("File"));
@@ -88,15 +87,14 @@ FileTree::FileTree()
     column->pack_start(*manage(cell_filename));
 
     column->add_attribute(cell_filename->property_text(), model_columns.filename);
-    column->set_cell_data_func(*cell_icon,     sigc::mem_fun(*this, &FileTree::icon_cell_data_func));
-    column->set_cell_data_func(*cell_filename, sigc::mem_fun(*this, &FileTree::text_cell_data_func));
+    column->set_cell_data_func(*cell_icon,     mem_fun(*this, &FileTree::icon_cell_data_func));
+    column->set_cell_data_func(*cell_filename, mem_fun(*this, &FileTree::text_cell_data_func));
 
     column->set_resizable(true);
     column->set_expand(true);
 
     column->set_sort_column(model_columns.collatekey);
   }
-
   {
     Column *const column = new Column(_("#"));
     append_column(*manage(column));
@@ -105,7 +103,7 @@ FileTree::FileTree()
     column->pack_start(*manage(cell_matchcount));
 
     column->add_attribute(cell_matchcount->property_text(), model_columns.matchcount);
-    column->set_cell_data_func(*cell_matchcount, sigc::mem_fun(*this, &FileTree::text_cell_data_func));
+    column->set_cell_data_func(*cell_matchcount, mem_fun(*this, &FileTree::text_cell_data_func));
 
     column->set_alignment(1.0);
     cell_matchcount->property_xalign() = 1.0;
@@ -118,10 +116,10 @@ FileTree::FileTree()
   const Glib::RefPtr<TreeSelection> selection = get_selection();
 
   selection->set_select_function(&FileTree::select_func);
-  selection->signal_changed().connect(sigc::mem_fun(*this, &FileTree::on_selection_changed));
+  selection->signal_changed().connect(mem_fun(*this, &FileTree::on_selection_changed));
 
   Gnome::Conf::Client::get_default_client()
-      ->signal_value_changed().connect(sigc::mem_fun(*this, &FileTree::on_conf_value_changed));
+      ->signal_value_changed().connect(mem_fun(*this, &FileTree::on_conf_value_changed));
 }
 
 FileTree::~FileTree()
@@ -153,7 +151,7 @@ void FileTree::find_files(const std::string& dirname, Pcre::Pattern& pattern,
   // slightly.  This in turn confuses TreeView::set_cursor() -- the first
   // call after the tree was completely filled just doesn't scroll.
   if (toplevel_.file_count > 0)
-    scroll_to_row(Gtk::TreePath(1U, 0));
+    scroll_to_row(Gtk::TreeModel::Path(1u, 0));
 
   signal_bound_state_changed(); // emit
 
@@ -232,7 +230,7 @@ BoundState FileTree::get_bound_state()
   {
     if (const Gtk::TreeModel::iterator iter = get_selection()->get_selected())
     {
-      Gtk::TreePath path (iter);
+      Gtk::TreeModel::Path path (iter);
 
       if (path > path_match_first_)
         bound &= ~BOUND_FIRST;
@@ -306,8 +304,8 @@ void FileTree::on_style_changed(const Glib::RefPtr<Gtk::Style>& previous_style)
 {
   const Glib::ustring detail = "regexxer-filetree";
 
-  pixbuf_directory_   = render_icon(Gtk::Stock::OPEN,          Gtk::ICON_SIZE_MENU, detail);
-  pixbuf_file_        = render_icon(Gtk::Stock::NEW,           Gtk::ICON_SIZE_MENU, detail);
+  pixbuf_directory_   = render_icon(Gtk::Stock::DIRECTORY,     Gtk::ICON_SIZE_MENU, detail);
+  pixbuf_file_        = render_icon(Gtk::Stock::FILE,          Gtk::ICON_SIZE_MENU, detail);
   pixbuf_load_failed_ = render_icon(Gtk::Stock::MISSING_IMAGE, Gtk::ICON_SIZE_MENU, detail);
 
   color_load_failed_ = get_style()->get_text(Gtk::STATE_INSENSITIVE);
@@ -368,10 +366,11 @@ void FileTree::text_cell_data_func(Gtk::CellRenderer* cell, const Gtk::TreeModel
 }
 
 // static
-bool FileTree::select_func(const Glib::RefPtr<Gtk::TreeModel>& model, const Gtk::TreePath& path, bool)
+bool FileTree::select_func(const Glib::RefPtr<Gtk::TreeModel>& model,
+                           const Gtk::TreeModel::Path& path, bool)
 {
   // Don't allow selection of directory nodes.
-  return get_fileinfo_from_iter(model->get_iter(path));
+  return (get_fileinfo_from_iter(model->get_iter(path)) != 0);
 }
 
 void FileTree::find_recursively(const std::string& dirname, FindData& find_data)
@@ -388,12 +387,12 @@ void FileTree::find_recursively(const std::string& dirname, FindData& find_data)
       if (signal_pulse()) // emit
         break;
 
-      const std::string basename = *pos;
+      const std::string filename = *pos;
 
-      if (!find_data.hidden && *basename.begin() == '.')
+      if (!find_data.hidden && *filename.begin() == '.')
         continue;
 
-      const std::string fullname = build_filename(dirname, basename);
+      const std::string fullname = build_filename(dirname, filename);
 
       if (file_test(fullname, FILE_TEST_IS_SYMLINK))
         continue; // ignore symbolic links
@@ -404,17 +403,16 @@ void FileTree::find_recursively(const std::string& dirname, FindData& find_data)
         // immediately.  The corresponding node will be created on demand if
         // there's actually a matching file in the directory or one of its
         // subdirectories.
-        //
-        ScopedPushDir pushdir (find_data.dirstack, basename);
+        ScopedPushDir pushdir (find_data.dirstack, fullname);
         find_recursively(fullname, find_data); // recurse
       }
       else if (file_test(fullname, FILE_TEST_IS_REGULAR))
       {
-        const ustring basename_utf8 = Util::filename_to_utf8_fallback(basename);
+        const ustring basename = Glib::filename_display_basename(fullname);
 
-        if (find_data.pattern.match(basename_utf8) > 0)
+        if (find_data.pattern.match(basename) > 0)
         {
-          find_add_file(basename_utf8, fullname, find_data);
+          find_add_file(basename, fullname, find_data);
           ++file_count;
         }
       }
@@ -473,7 +471,7 @@ void FileTree::find_fill_dirstack(FindData& find_data)
     if (pdir->second) // node already created
       continue;
 
-    const Glib::ustring dirname = Util::filename_to_utf8_fallback(pdir->first);
+    const Glib::ustring dirname = Glib::filename_display_basename(pdir->first);
 
     // Build the collate key with a leading '0' so that directories always
     // come first.  This is simpler and faster than explicitely checking for
@@ -529,8 +527,8 @@ bool FileTree::save_file_at_iter(const Gtk::TreeModel::iterator& iter,
     }
     catch (const Glib::Error& error)
     {
-      error_list->push_back(Util::compose(_("Failed to save file \"%1\": %2"),
-                                          Util::filename_to_utf8_fallback(fileinfo->fullname),
+      error_list->push_back(Util::compose(_("Failed to save file \342\200\234%1\342\200\235: %2"),
+                                          Glib::filename_display_basename(fileinfo->fullname),
                                           error.what()));
     }
 
@@ -565,7 +563,15 @@ bool FileTree::find_matches_at_path_iter(const Gtk::TreeModel::Path& path,
     Util::ScopedConnection conn (buffer->signal_pulse.connect(signal_pulse.make_slot()));
 
     const int old_match_count = buffer->get_match_count();
-    const int new_match_count = buffer->find_matches(find_data.pattern, find_data.multiple);
+
+    // Optimize the common case and construct the feedback slot only if there
+    // are actually any handlers connected to the signal.  find_matches() can
+    // then check whether the slot is empty to avoid providing arguments that
+    // are never going to be used.
+    const int new_match_count =
+        buffer->find_matches(find_data.pattern, find_data.multiple, (signal_feedback.empty())
+                             ? sigc::slot<void, int, const Glib::ustring&>()
+                             : sigc::bind(signal_feedback.make_slot(), fileinfo));
 
     if (new_match_count > 0)
     {
@@ -619,9 +625,8 @@ bool FileTree::replace_matches_at_path_iter(const Gtk::TreeModel::Path& path,
         // a single user action object for all replacements in all buffers.
         // Note that the caller must block conn_undo_stack_push_ to avoid
         // double notification.
-        Util::ScopedConnection conn1 (buffer->signal_undo_stack_push.
-                                      connect(replace_data.slot_undo_stack_push));
-
+        Util::ScopedConnection conn1 (buffer->signal_undo_stack_push
+                                        .connect(replace_data.slot_undo_stack_push));
         Util::ScopedConnection conn2 (buffer->signal_pulse.connect(signal_pulse.make_slot()));
 
         buffer->replace_all_matches(replace_data.substitution);
@@ -639,7 +644,7 @@ bool FileTree::replace_matches_at_path_iter(const Gtk::TreeModel::Path& path,
   return false;
 }
 
-void FileTree::expand_and_select(const Gtk::TreePath& path)
+void FileTree::expand_and_select(const Gtk::TreeModel::Path& path)
 {
   expand_to_path(path);
   set_cursor(path);
@@ -725,7 +730,7 @@ void FileTree::on_selection_changed()
           connect(sigc::mem_fun(*this, &FileTree::on_buffer_undo_stack_push));
     }
 
-    last_selected_rowref_.reset(new TreeRowRef(treestore_, Gtk::TreePath(iter)));
+    last_selected_rowref_.reset(new TreeRowRef(treestore_, Gtk::TreeModel::Path(iter)));
   }
 
   if (last_selected_ && last_selected_ != fileinfo &&
@@ -788,7 +793,7 @@ void FileTree::on_buffer_match_count_changed()
     // The range should've been set up already since old_sum_matches > 0.
     g_return_if_fail(path_match_first_ <= path_match_last_);
 
-    Gtk::TreePath path (iter);
+    Gtk::TreeModel::Path path (iter);
 
     if (old_match_count == 0)
     {
@@ -896,7 +901,7 @@ void FileTree::propagate_modified_change(const Gtk::TreeModel::iterator& pos, bo
   const int difference = (modified) ? 1 : -1;
   const FileTreeColumns& columns = FileTreeColumns::instance();
 
-  Gtk::TreePath path (pos);
+  Gtk::TreeModel::Path path (pos);
   treestore_->row_changed(path, pos);
 
   for (Gtk::TreeModel::iterator iter = pos->parent(); iter && path.up(); iter = iter->parent())
@@ -938,14 +943,14 @@ void FileTree::load_file_with_fallback(const Gtk::TreeModel::iterator& iter,
 
     fileinfo->buffer = FileBuffer::create_with_error_message(
         render_icon(Gtk::Stock::DIALOG_ERROR, Gtk::ICON_SIZE_DIALOG),
-        Util::compose(_("\"%1\" seems to be a binary file."), filename));
+        Util::compose(_("\342\200\234%1\342\200\235 seems to be a binary file."), filename));
   }
 
   if (old_load_failed != fileinfo->load_failed)
   {
     // Trigger signal_row_changed() because the value of fileinfo->load_failed
     // changed, which means we have to change icon and color of the row.
-    treestore_->row_changed(Gtk::TreePath(iter), iter);
+    treestore_->row_changed(Gtk::TreeModel::Path(iter), iter);
   }
 }
 
@@ -959,4 +964,3 @@ void FileTree::on_conf_value_changed(const Glib::ustring& key, const Gnome::Conf
 }
 
 } // namespace Regexxer
-
