@@ -1,21 +1,21 @@
-/* $Id: filetreeprivate.cc,v 1.7 2004/06/19 23:52:07 daniel Exp $
+/*
+ * Copyright (c) 2002-2007  Daniel Elstner  <daniel.kitta@gmail.com>
  *
- * Copyright (c) 2002  Daniel Elstner  <daniel.elstner@gmx.net>
+ * This file is part of regexxer.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License VERSION 2 as
- * published by the Free Software Foundation.  You are not allowed to
- * use any other version of the license; unless you got the explicit
- * permission from the author to do so.
+ * regexxer is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * regexxer is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with regexxer; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "filetreeprivate.h"
@@ -23,7 +23,6 @@
 #include <glib.h>
 #include <gtkmm/treestore.h>
 
-
 namespace Regexxer
 {
 
@@ -37,27 +36,29 @@ const FileTreeColumns& FileTreeColumns::instance()
   return column_record;
 }
 
-int default_sort_func(const Gtk::TreeModel::iterator& lhs, const Gtk::TreeModel::iterator& rhs)
+int default_sort_func(const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b)
 {
   const FileTreeColumns& columns = FileTreeColumns::instance();
 
-  const std::string lhs_key = (*lhs)[columns.collatekey];
-  const std::string rhs_key = (*rhs)[columns.collatekey];
+  const std::string a_key = (*a)[columns.collatekey];
+  const std::string b_key = (*b)[columns.collatekey];
 
-  return lhs_key.compare(rhs_key);
+  return a_key.compare(b_key);
 }
 
-int collatekey_sort_func(const Gtk::TreeModel::iterator& lhs, const Gtk::TreeModel::iterator& rhs)
+int collatekey_sort_func(const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b)
 {
+  typedef std::string::difference_type diff_type;
+
   const FileTreeColumns& columns = FileTreeColumns::instance();
 
-  const std::string lhs_key = (*lhs)[columns.collatekey];
-  const std::string rhs_key = (*rhs)[columns.collatekey];
+  const std::string a_key = (*a)[columns.collatekey];
+  const std::string b_key = (*b)[columns.collatekey];
 
-  if (lhs_key.size() > 1 && rhs_key.size() > 1)
-    return lhs_key.compare(1, std::string::npos, rhs_key, 1, std::string::npos);
+  if (a_key.size() > 1 && b_key.size() > 1)
+    return a_key.compare(1, std::string::npos, b_key, 1, std::string::npos);
   else
-    return (lhs_key.size() - rhs_key.size());
+    return diff_type(a_key.size()) - diff_type(b_key.size());
 }
 
 bool next_match_file(Gtk::TreeModel::iterator& iter, Gtk::TreeModel::Path* collapse)
@@ -66,8 +67,9 @@ bool next_match_file(Gtk::TreeModel::iterator& iter, Gtk::TreeModel::Path* colla
 
   const FileTreeColumns& columns = FileTreeColumns::instance();
   Gtk::TreeModel::iterator parent = iter->parent();
+  ++iter;
 
-  for (++iter;;)
+  for (;;)
   {
     if (iter)
     {
@@ -119,7 +121,7 @@ bool prev_match_file(Gtk::TreeModel::iterator& iter, Gtk::TreeModel::Path* colla
         if (const Gtk::TreeModel::Children& children = iter->children()) // directory?
         {
           parent = iter;
-          path.push_back(children.size());
+          path.push_back(children.size()); // one beyond the last child
           continue;
         }
 
@@ -143,10 +145,10 @@ bool prev_match_file(Gtk::TreeModel::iterator& iter, Gtk::TreeModel::Path* colla
 
 } // namespace FileTreePrivate
 
-
 /**** Regexxer::FileTree::TreeRowRef ***************************************/
 
-FileTree::TreeRowRef::TreeRowRef(const Glib::RefPtr<Gtk::TreeModel>& model, const Gtk::TreePath& path)
+FileTree::TreeRowRef::TreeRowRef(const Glib::RefPtr<Gtk::TreeModel>& model,
+                                 const Gtk::TreeModel::Path& path)
 :
   Gtk::TreeRowReference(model, path)
 {}
@@ -154,7 +156,6 @@ FileTree::TreeRowRef::TreeRowRef(const Glib::RefPtr<Gtk::TreeModel>& model, cons
 FileTree::TreeRowRef::~TreeRowRef()
 {}
 
-
 /**** Regexxer::FileTree::MessageList **************************************/
 
 FileTree::MessageList::MessageList()
@@ -163,7 +164,6 @@ FileTree::MessageList::MessageList()
 FileTree::MessageList::~MessageList()
 {}
 
-
 /**** Regexxer::FileTree::Error ********************************************/
 
 FileTree::Error::Error(const Util::SharedPtr<FileTree::MessageList>& error_list)
@@ -190,21 +190,19 @@ const std::list<Glib::ustring>& FileTree::Error::get_error_list() const
   return *error_list_;
 }
 
-
 /**** Regexxer::FileTree::FindData *****************************************/
 
 FileTree::FindData::FindData(Pcre::Pattern& pattern_, bool recursive_, bool hidden_)
 :
-  pattern     (pattern_),
-  recursive   (recursive_),
-  hidden      (hidden_),
-  error_list  (new FileTree::MessageList())
+  pattern    (pattern_),
+  recursive  (recursive_),
+  hidden     (hidden_),
+  error_list (new FileTree::MessageList())
 {}
 
 FileTree::FindData::~FindData()
 {}
 
-
 /**** Regexxer::FileTree::FindMatchesData **********************************/
 
 FileTree::FindMatchesData::FindMatchesData(Pcre::Pattern& pattern_, bool multiple_)
@@ -214,7 +212,6 @@ FileTree::FindMatchesData::FindMatchesData(Pcre::Pattern& pattern_, bool multipl
   path_match_first_set (false)
 {}
 
-
 /**** Regexxer::FileTree::ReplaceMatchesData *******************************/
 
 FileTree::ReplaceMatchesData::ReplaceMatchesData(FileTree& filetree_,
@@ -236,7 +233,6 @@ void FileTree::ReplaceMatchesData::undo_stack_push(UndoActionPtr undo_action)
   undo_stack->push(UndoActionPtr(new BufferActionShell(filetree, row_reference, undo_action)));
 }
 
-
 /**** Regexxer::FileTree::ScopedBlockSorting *******************************/
 
 FileTree::ScopedBlockSorting::ScopedBlockSorting(FileTree& filetree)
@@ -263,7 +259,6 @@ FileTree::ScopedBlockSorting::~ScopedBlockSorting()
   filetree_.set_headers_clickable(true);
 }
 
-
 /**** Regexxer::FileTree::BufferActionShell ********************************/
 
 FileTree::BufferActionShell::BufferActionShell(FileTree& filetree,
@@ -282,10 +277,10 @@ bool FileTree::BufferActionShell::do_undo(const sigc::slot<bool>& pulse)
 {
   g_return_val_if_fail(row_reference_->is_valid(), false);
 
-  const Gtk::TreePath path (row_reference_->get_path());
+  const Gtk::TreeModel::Path path = row_reference_->get_path();
 
-  if (!filetree_.last_selected_rowref_ ||
-      filetree_.last_selected_rowref_->get_path() != path)
+  if (!filetree_.last_selected_rowref_
+      || filetree_.last_selected_rowref_->get_path() != path)
   {
     filetree_.expand_and_select(path);
   }
@@ -294,4 +289,3 @@ bool FileTree::BufferActionShell::do_undo(const sigc::slot<bool>& pulse)
 }
 
 } // namespace Regexxer
-
