@@ -1,25 +1,24 @@
-/* $Id: mainwindow.cc,v 1.90 2004/06/29 19:56:34 daniel Exp $
+/*
+ * Copyright (c) 2002-2007  Daniel Elstner  <daniel.kitta@gmail.com>
  *
- * Copyright (c) 2004  Daniel Elstner  <daniel.elstner@gmx.net>
+ * This file is part of regexxer.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License VERSION 2 as
- * published by the Free Software Foundation.  You are not allowed to
- * use any other version of the license; unless you got the explicit
- * permission from the author to do so.
+ * regexxer is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * regexxer is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with regexxer; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "mainwindow.h"
-#include "aboutdialog.h"
 #include "filetree.h"
 #include "globalstrings.h"
 #include "pcreshell.h"
@@ -29,17 +28,15 @@
 #include "translation.h"
 
 #include <glib.h>
-#include <gtk/gtktooltips.h>  /* XXX: see load_xml() */
 #include <gtkmm.h>
-#include <gtkmm/comboboxentry.h>
 #include <gconfmm/client.h>
 #include <libglademm/xml.h>
 #include <algorithm>
 #include <functional>
+#include <iostream>
 
 #include <config.h>
 
-
 namespace
 {
 
@@ -47,8 +44,32 @@ enum { BUSY_GUI_UPDATE_INTERVAL = 16 };
 
 typedef Glib::RefPtr<Regexxer::FileBuffer> FileBufferPtr;
 
-const char *const selection_clipboard = "CLIPBOARD";
+static const char *const selection_clipboard = "CLIPBOARD";
 
+/*
+ * List of authors to be displayed in the about dialog.
+ */
+static const char *const program_authors[] =
+{
+  "Daniel Elstner <daniel.kitta@gmail.com>",
+  "Murray Cumming <murrayc@murrayc.com>",
+  0
+};
+
+static const char *const program_license =
+  "regexxer is free software; you can redistribute it and/or modify "
+  "it under the terms of the GNU General Public License as published by "
+  "the Free Software Foundation; either version 2 of the License, or "
+  "(at your option) any later version.\n"
+  "\n"
+  "regexxer is distributed in the hope that it will be useful, "
+  "but WITHOUT ANY WARRANTY; without even the implied warranty of "
+  "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the "
+  "GNU General Public License for more details.\n"
+  "\n"
+  "You should have received a copy of the GNU General Public License "
+  "along with regexxer; if not, write to the Free Software Foundation, "
+  "Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA\n";
 
 class FileErrorDialog : public Gtk::MessageDialog
 {
@@ -98,12 +119,45 @@ FileErrorDialog::FileErrorDialog(Gtk::Window& parent, const Glib::ustring& messa
 FileErrorDialog::~FileErrorDialog()
 {}
 
-} // anonymous namespace
+static
+void print_location(int linenumber, const Glib::ustring& subject, Regexxer::FileInfoPtr fileinfo)
+{
+  std::cout << fileinfo->fullname << ':' << linenumber + 1 << ':';
 
+  std::string charset;
+
+  if (Glib::get_charset(charset))
+    std::cout << subject.raw(); // charset is UTF-8
+  else
+    std::cout << Glib::convert_with_fallback(subject.raw(), charset, "UTF-8");
+
+  std::cout << std::endl;
+}
+
+} // anonymous namespace
 
 namespace Regexxer
 {
 
+/**** Regexxer::InitState **************************************************/
+
+InitState::InitState()
+:
+  folder        (),
+  pattern       (),
+  regex         (),
+  substitution  (),
+  no_recursive  (false),
+  hidden        (false),
+  no_global     (false),
+  ignorecase    (false),
+  feedback      (false),
+  no_autorun    (false)
+{}
+
+InitState::~InitState()
+{}
+
 /**** Regexxer::MainWindow::BusyAction *************************************/
 
 class MainWindow::BusyAction
@@ -121,13 +175,12 @@ public:
   ~BusyAction() { object_.busy_action_leave(); }
 };
 
-
 /**** Regexxer::MainWindow *************************************************/
 
 MainWindow::MainWindow()
 :
   toolbar_                (0),
-  entry_folder_           (0),
+  button_folder_          (0),
   entry_pattern_          (0),
   button_recursive_       (0),
   button_hidden_          (0),
@@ -149,17 +202,47 @@ MainWindow::MainWindow()
   textview_->set_buffer(FileBuffer::create());
   window_->set_title(PACKAGE_NAME);
 
-  entry_folder_->set_text(
-      Util::filename_to_utf8_fallback(Util::shorten_pathname(Glib::get_current_dir())));
-
   connect_signals();
-
-  entry_pattern_->set_text("*");
 }
 
 MainWindow::~MainWindow()
 {}
 
+void MainWindow::initialize(const InitState& init)
+{
+  std::string folder;
+
+  if (!init.folder.empty())
+    folder = init.folder.front();
+  if (!Glib::path_is_absolute(folder))
+    folder = Glib::build_filename(Glib::get_current_dir(), folder);
+
+  const bool folder_exists = button_folder_->set_current_folder(folder);
+
+  entry_pattern_->set_text((init.pattern.empty()) ? Glib::ustring(1, '*') : init.pattern);
+  entry_regex_  ->set_text(init.regex);
+  entry_substitution_->set_text(init.substitution);
+
+  button_recursive_->set_active(!init.no_recursive);
+  button_hidden_   ->set_active(init.hidden);
+  button_multiple_ ->set_active(!init.no_global);
+  button_caseless_ ->set_active(init.ignorecase);
+
+  if (init.feedback)
+    filetree_->signal_feedback.connect(&print_location);
+
+  // Strangely, folder_exists seems to be always true, probably because the
+  // file chooser works asynchronously but the GLib main loop isn't running
+  // yet.  As a work-around, explicitely check whether the directory exists
+  // on the file system as well.
+  if (folder_exists && !init.no_autorun
+      && !init.folder.empty() && !init.pattern.empty()
+      && Glib::file_test(folder, Glib::FILE_TEST_IS_DIR))
+  {
+    Glib::signal_idle().connect(sigc::mem_fun(*this, &MainWindow::autorun_idle));
+  }
+}
+
 /**** Regexxer::MainWindow -- private **************************************/
 
 void MainWindow::load_xml()
@@ -172,7 +255,7 @@ void MainWindow::load_xml()
   window_.reset(xml->get_widget("mainwindow", mainwindow));
 
   xml->get_widget("toolbar",             toolbar_);
-  xml->get_widget("entry_folder",        entry_folder_);
+  xml->get_widget("button_folder",       button_folder_);
   xml->get_widget("button_recursive",    button_recursive_);
   xml->get_widget("button_hidden",       button_hidden_);
   xml->get_widget("entry_regex",         entry_regex_);
@@ -183,27 +266,7 @@ void MainWindow::load_xml()
   xml->get_widget("textview",            textview_);
   xml->get_widget("entry_preview",       entry_preview_);
   xml->get_widget("statusline",          statusline_);
-
-  Gtk::ComboBoxEntry* combo_pattern = 0;
-  xml->get_widget("combo_pattern", combo_pattern);
-  entry_pattern_ = dynamic_cast<Gtk::Entry*>(combo_pattern->get_child());
-
-  // Current libglade does not yet provide access to the internal child of
-  // GtkComboBoxEntry.  However, I want a tooltip to be assigned to the entry
-  // and not to the combo box as a whole (by means of GtkEventBox).  I hope
-  // libglade will be fixed soon -- until then, this hack assigns the tooltip
-  // manually using the group created by libglade.
-
-  if (GtkTooltipsData *const tipdata = gtk_tooltips_data_get(entry_folder_->Gtk::Widget::gobj()))
-  {
-    gtk_tooltips_set_tip(tipdata->tooltips, entry_pattern_->Gtk::Widget::gobj(),
-                         _("A filename pattern as used by the shell. Character classes "
-                           "[ab] and csh style brace expressions {a,b} are supported."), 0);
-  }
-
-  Gtk::Button* button_folder = 0;
-  xml->get_widget("button_folder", button_folder);
-  button_folder->signal_clicked().connect(sigc::mem_fun(*this, &MainWindow::on_select_folder));
+  xml->get_widget("combo_pattern_entry", entry_pattern_);
 
   controller_.load_xml(xml);
 }
@@ -217,7 +280,6 @@ void MainWindow::connect_signals()
   window_->signal_style_changed().connect(mem_fun(*this, &MainWindow::on_style_changed));
   window_->signal_delete_event ().connect(mem_fun(*this, &MainWindow::on_delete_event));
 
-  entry_folder_ ->signal_activate().connect(controller_.find_files.slot());
   entry_pattern_->signal_activate().connect(controller_.find_files.slot());
   entry_pattern_->signal_changed ().connect(mem_fun(*this, &MainWindow::on_entry_pattern_changed));
 
@@ -273,6 +335,16 @@ void MainWindow::connect_signals()
       mem_fun(*this, &MainWindow::on_undo_stack_push));
 }
 
+bool MainWindow::autorun_idle()
+{
+  controller_.find_files.activate();
+
+  if (!busy_action_cancel_ && entry_regex_->get_text_length() > 0)
+    controller_.find_matches.activate();
+
+  return false;
+}
+
 void MainWindow::on_hide()
 {
   on_busy_action_cancel();
@@ -281,7 +353,6 @@ void MainWindow::on_hide()
   // necessary since they'd be deleted in the destructor anyway.  But if we
   // have to do a lot of cleanup the dialogs would stay open for that time,
   // which doesn't look neat.
-
   {
     // Play safe and transfer ownership, and let the dtor do the delete.
     const std::auto_ptr<Gtk::Dialog> temp (about_dialog_);
@@ -338,7 +409,8 @@ bool MainWindow::confirm_quit_request()
   if (filetree_->get_modified_count() == 0)
     return true;
 
-  Gtk::MessageDialog dialog (*window_, _("Some files haven't been saved yet.\nQuit anyway?"),
+  Gtk::MessageDialog dialog (*window_,
+                             _("Some files haven\342\200\231t been saved yet.\nQuit anyway?"),
                              false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE, true);
 
   dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
@@ -347,75 +419,24 @@ bool MainWindow::confirm_quit_request()
   return (dialog.run() == Gtk::RESPONSE_OK);
 }
 
-/*
- * Expand the pathname entered into the folder entry, and convert it to
- * the local filename encoding.  If the conversion fails, show an error
- * message to the user and return an empty string.
- */
-std::string MainWindow::get_folder_fullname() const
-{
-  const Glib::ustring folder = entry_folder_->get_text();
-
-  if (folder.empty())
-    return Glib::get_current_dir();
-
-  try
-  {
-    return Util::expand_pathname(Glib::filename_from_utf8(folder));
-  }
-  catch (const Glib::ConvertError&)
-  {
-    const Glib::ustring message = Util::compose(
-        _("The folder name \"%1\" contains characters not representable "
-          "in the encoding of the local file system."), folder);
-
-    Gtk::MessageDialog dialog (*window_, message, false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
-    dialog.run();
-
-    return std::string();
-  }
-}
-
-void MainWindow::on_select_folder()
-{
-  using namespace Gtk;
-
-  const std::string folder = get_folder_fullname();
-
-  if (folder.empty())
-    return;
-
-  FileChooserDialog chooser (*window_, _("Select a folder"), FILE_CHOOSER_ACTION_SELECT_FOLDER);
-
-  chooser.add_button(Stock::CANCEL, RESPONSE_CANCEL);
-  chooser.add_button(Stock::OK,     RESPONSE_OK);
-  chooser.set_default_response(RESPONSE_OK);
-  chooser.set_modal(true);
-  chooser.set_local_only(true);
-  chooser.set_current_folder(folder);
-
-  if (chooser.run() == RESPONSE_OK)
-  {
-    const std::string shortname = Util::shorten_pathname(chooser.get_filename());
-    entry_folder_->set_text(Util::filename_to_utf8_fallback(shortname));
-  }
-}
-
 void MainWindow::on_find_files()
 {
   if (filetree_->get_modified_count() > 0)
   {
-    Gtk::MessageDialog dialog (*window_, _("Some files haven't been saved yet.\nContinue anyway?"),
+    Gtk::MessageDialog dialog (*window_,
+                               _("Some files haven\342\200\231t been saved yet.\nContinue anyway?"),
                                false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK_CANCEL, true);
 
     if (dialog.run() != Gtk::RESPONSE_OK)
       return;
   }
 
-  const std::string folder = get_folder_fullname();
+  std::string folder = button_folder_->get_filename();
 
   if (folder.empty())
-    return;
+    folder = Glib::get_current_dir();
+
+  g_return_if_fail(Glib::path_is_absolute(folder));
 
   undo_stack_clear();
 
@@ -456,19 +477,17 @@ void MainWindow::on_exec_search()
   try
   {
     Pcre::Pattern pattern (regex, (caseless) ? Pcre::CASELESS : Pcre::CompileOptions(0));
+
     filetree_->find_matches(pattern, multiple);
   }
   catch (const Pcre::Error& error)
   {
-    const int offset = error.offset();
-    const Glib::ustring message = (offset >= 0 && unsigned(offset) < regex.length())
-      ? Util::compose(_("Error in regular expression at \"%1\" (index %2):\n%3"),
-                      regex.substr(offset, 1), Util::int_to_string(offset + 1), error.what())
-      : Util::compose(_("Error in regular expression:\n%1"), error.what());
-
-    Gtk::MessageDialog dialog (*window_, message, false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
+    Gtk::MessageDialog dialog (*window_, error.what(), false,
+                               Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
     dialog.run();
 
+    const int offset = error.offset();
+
     if (offset >= 0 && offset < entry_regex_->get_text_length())
     {
       entry_regex_->grab_focus();
@@ -723,8 +742,8 @@ void MainWindow::update_preview()
   {
     Glib::ustring preview;
     const int pos = buffer->get_line_preview(entry_substitution_->get_text(), preview);
-    entry_preview_->set_text(preview);
 
+    entry_preview_->set_text(preview);
     controller_.replace.set_enabled(pos >= 0);
 
     // Beware, strange code ahead!
@@ -759,10 +778,10 @@ void MainWindow::update_preview()
 
 void MainWindow::set_title_filename(const std::string& filename)
 {
-  Glib::ustring title = Util::filename_to_utf8_fallback(Glib::path_get_basename(filename));
+  Glib::ustring title = Glib::filename_display_basename(filename);
 
   title += " (";
-  title += Util::filename_to_utf8_fallback(Util::shorten_pathname(Glib::path_get_dirname(filename)));
+  title += Util::filename_short_display_name(Glib::path_get_dirname(filename));
   title += ") \342\200\223 " PACKAGE_NAME; // U+2013 EN DASH
 
   window_->set_title(title);
@@ -786,7 +805,6 @@ void MainWindow::busy_action_leave()
   g_return_if_fail(busy_action_running_);
 
   busy_action_running_ = false;
-  busy_action_cancel_  = false;
 
   statusline_->pulse_stop();
 
@@ -824,16 +842,28 @@ void MainWindow::on_about()
   }
   else
   {
-    std::auto_ptr<Gtk::Dialog> dialog = AboutDialog::create(*window_);
+    std::auto_ptr<Gtk::AboutDialog> dialog (new Gtk::AboutDialog());
 
-    dialog->signal_hide().connect(sigc::mem_fun(*this, &MainWindow::on_about_dialog_hide));
+    dialog->set_version(PACKAGE_VERSION);
+    dialog->set_logo_icon_name(PACKAGE_TARNAME);
+    dialog->set_comments(_("Search and replace using regular expressions"));
+    dialog->set_copyright("Copyright \302\251 2002-2007 Daniel Elstner");
+    dialog->set_website("http://regexxer.sourceforge.net/");
+
+    dialog->set_authors(program_authors);
+    dialog->set_translator_credits(_("translator-credits"));
+    dialog->set_license(program_license);
+    dialog->set_wrap_license(true);
+
+    dialog->set_transient_for(*window_);
     dialog->show();
+    dialog->signal_response().connect(sigc::mem_fun(*this, &MainWindow::on_about_dialog_response));
 
     about_dialog_ = dialog;
   }
 }
 
-void MainWindow::on_about_dialog_hide()
+void MainWindow::on_about_dialog_response(int)
 {
   // Play safe and transfer ownership, and let the dtor do the delete.
   const std::auto_ptr<Gtk::Dialog> temp (about_dialog_);
@@ -849,7 +879,8 @@ void MainWindow::on_preferences()
   {
     std::auto_ptr<PrefDialog> dialog (new PrefDialog(*window_));
 
-    dialog->get_dialog()->signal_hide().connect(sigc::mem_fun(*this, &MainWindow::on_pref_dialog_hide));
+    dialog->get_dialog()->signal_hide()
+        .connect(sigc::mem_fun(*this, &MainWindow::on_pref_dialog_hide));
     dialog->get_dialog()->show();
 
     pref_dialog_ = dialog;
@@ -880,4 +911,3 @@ void MainWindow::on_conf_value_changed(const Glib::ustring& key, const Gnome::Co
 }
 
 } // namespace Regexxer
-
