@@ -1,21 +1,21 @@
-/* $Id: prefdialog.cc,v 1.31 2004/06/30 00:07:33 daniel Exp $
+/*
+ * Copyright (c) 2002-2007  Daniel Elstner  <daniel.kitta@gmail.com>
  *
- * Copyright (c) 2004  Daniel Elstner  <daniel.elstner@gmx.net>
+ * This file is part of regexxer.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License VERSION 2 as
- * published by the Free Software Foundation.  You are not allowed to
- * use any other version of the license; unless you got the explicit
- * permission from the author to do so.
+ * regexxer is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * regexxer is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with regexxer; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "prefdialog.h"
@@ -31,10 +31,10 @@
 
 #include <config.h>
 
-
 namespace
 {
 
+static
 const Gtk::ToolbarStyle toolbar_style_values[] =
 {
   Gtk::TOOLBAR_ICONS,
@@ -43,6 +43,7 @@ const Gtk::ToolbarStyle toolbar_style_values[] =
   Gtk::TOOLBAR_BOTH_HORIZ
 };
 
+static
 int get_toolbar_style_index(const Glib::ustring& value)
 {
   const Gtk::ToolbarStyle toolbar_style = Util::enum_from_nick<Gtk::ToolbarStyle>(value);
@@ -58,7 +59,6 @@ int get_toolbar_style_index(const Glib::ustring& value)
 
 } // anonymous namespace
 
-
 namespace Regexxer
 {
 
@@ -182,7 +182,7 @@ void PrefDialog::initialize_configuration()
   using namespace Gnome::Conf;
 
   const Glib::RefPtr<Client> client = Client::get_default_client();
-  const std::list<Entry> entries (client->all_entries(REGEXXER_GCONF_DIRECTORY));
+  const std::list<Entry> entries (client->all_entries(conf_dir_application));
 
   for (std::list<Entry>::const_iterator p = entries.begin(); p != entries.end(); ++p)
   {
@@ -245,8 +245,9 @@ void PrefDialog::on_entry_fallback_activate()
   }
   else
   {
-    const Glib::ustring message = Util::compose(_("\"%1\" is not a valid encoding."),
-                                                fallback_encoding);
+    const Glib::ustring message =
+        Util::compose(_("\342\200\234%1\342\200\235 is not a valid encoding."), fallback_encoding);
+
     Gtk::MessageDialog error_dialog (*dialog_, message, false,
                                      Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
     error_dialog.run();
@@ -254,4 +255,3 @@ void PrefDialog::on_entry_fallback_activate()
 }
 
 } // namespace Regexxer
-
