@@ -1,21 +1,21 @@
-/* $Id: stringutils.h,v 1.16 2004/06/09 01:26:11 daniel Exp $
+/*
+ * Copyright (c) 2002-2007  Daniel Elstner  <daniel.kitta@gmail.com>
  *
- * Copyright (c) 2002  Daniel Elstner  <daniel.elstner@gmx.net>
+ * This file is part of regexxer.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License VERSION 2 as
- * published by the Free Software Foundation.  You are not allowed to
- * use any other version of the license; unless you got the explicit
- * permission from the author to do so.
+ * regexxer is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * regexxer is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with regexxer; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef REGEXXER_STRINGUTILS_H_INCLUDED
@@ -29,11 +29,12 @@
 
 namespace Gdk { class Color; }
 
-
 namespace Util
 {
 
-typedef std::vector< std::pair<int,int> > CaptureVector;
+typedef std::vector< std::pair<int, int> > CaptureVector;
+
+Glib::ustring wstring_to_utf8(const std::wstring& str);
 
 bool validate_encoding(const std::string& encoding);
 bool encodings_equal(const std::string& lhs, const std::string& rhs);
@@ -43,13 +44,9 @@ Glib::ustring substitute_references(const Glib::ustring& substitution,
                                     const Glib::ustring& subject,
                                     const CaptureVector& captures);
 
-Glib::ustring filename_to_utf8_fallback(const std::string& filename);
-Glib::ustring convert_to_ascii(const std::string& str);
+Glib::ustring filename_short_display_name(const std::string& filename);
+
 Glib::ustring int_to_string(int number);
-
-std::string shorten_pathname(const std::string& path);
-std::string expand_pathname(const std::string& path);
-
 Glib::ustring color_to_string(const Gdk::Color& color);
 
 int enum_from_nick_impl(GType type, const Glib::ustring& nick);
@@ -66,4 +63,3 @@ Glib::ustring enum_to_nick(T value)
 } // namespace Util
 
 #endif /* REGEXXER_STRINGUTILS_H_INCLUDED */
-
